@@ -102,6 +102,7 @@ CZipContextMenu::CZipContextMenu():
    _isMenuForFM(false),
    _dropMode(false),
    _bitmap(NULL),
+   _writeZone((UInt32)(Int32)-1),
    IsSeparator(false),
    IsRoot(true),
    CurrentSubCommand(0)
@@ -560,6 +561,7 @@ STDMETHODIMP CZipContextMenu::QueryContextMenu(HMENU hMenu, UINT indexMenu,
   ci.Load();
 
   _elimDup = ci.ElimDup;
+  _writeZone = ci.WriteZone;
 
   HBITMAP bitmap = NULL;
   if (ci.MenuIcons.Val)
@@ -1167,7 +1169,8 @@ HRESULT CZipContextMenu::InvokeCommandCommon(const CCommandMapItem &cmi)
       {
         ExtractArchives(_fileNames, cmi.Folder,
             (cmdID == kExtract), // showDialog
-            (cmdID == kExtractTo) && _elimDup.Val // elimDup
+            (cmdID == kExtractTo) && _elimDup.Val, // elimDup
+            _writeZone
             );
         break;
       }
