@@ -1,5 +1,5 @@
 /* 7zipInstall.c - 7-Zip Installer
-2021-09-02 : Igor Pavlov : Public domain */
+2022-07-15 : Igor Pavlov : Public domain */
 
 #include "Precomp.h"
 
@@ -22,6 +22,10 @@
 
 #include "resource.h"
 
+#if defined(__GNUC__) && (__GNUC__ >= 8)
+  #pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif
+
 #define LLL_(quote) L##quote
 #define LLL(quote) LLL_(quote)
 
@@ -70,7 +74,7 @@ static LPCWSTR const k_Reg_Software_7zip = L"Software\\7-Zip";
   #endif
 #endif
 
-#define k_7zip_with_Ver  k_7zip_with_Ver_base k_Postfix 
+#define k_7zip_with_Ver  k_7zip_with_Ver_base k_Postfix
 
 
 static LPCWSTR const k_7zip_with_Ver_str = k_7zip_with_Ver;
